@@ -14,6 +14,23 @@ script_dir = Path(os.path.dirname(os.path.realpath(__file__)))
 cache_file = Path("fahrplan_cache")
 requests_cache.install_cache(str(script_dir / cache_file))
 
+cli_defaults = {
+    "speaker": None,
+    "title": None,
+    "track": None,
+    "day": -1,  # 0 seems to be a valid day value in some c3s
+    "start": None,
+    "room": "all",
+    "conference": None,
+    "show_abstract": False,
+    "show_description": False,
+    "sort": None,
+    "tablefmt": "fancy_grid",
+    "update_cache": False,
+    "no_past": False,
+    "column_width": 60,
+}
+
 
 class Colour:
     HEADER = "\033[95m"
@@ -27,7 +44,7 @@ class Colour:
 
 
 class Fahrplan:
-    def __init__(self, column_width, update_cache):
+    def __init__(self, update_cache: bool = cli_defaults['update_cache']):
         self.urls = [
             f"https://raw.githubusercontent.com/voc/{x}C3_schedule/master/everything.schedule.json"
             for x in range(32, 37)
@@ -35,7 +52,6 @@ class Fahrplan:
         self.urls.append("https://data.c3voc.de/rC3/everything.schedule.json")
         self.fahrplans = []
         self.flat_plans = []
-        self.column_width = column_width
         self.update_cache = update_cache
         self._get_fahrplans()
         self.flatten_fahrplans()
@@ -58,17 +74,6 @@ class Fahrplan:
             )
             sys.exit()
 
-    def format_row(self, row_content):
-        for key in row_content.keys():
-            cell_value = row_content[key]
-            if type(cell_value) == str and (value_length := len(cell_value)) >= self.column_width:
-                row_content[key] = " \n".join(
-                    # fmt: off
-                    cell_value[n:n + self.column_width]
-                    # fmt: on
-                    for n in range(0, value_length, self.column_width)
-                )
-
     def flatten_fahrplans(self):
         self.flat_plans = []
         for schedule in self.fahrplans:
@@ -101,7 +106,6 @@ class Fahrplan:
                                 ]
                             ),
                         }
-                        self.format_row(current_talk)
                         self.flat_plans.append(current_talk)
 
 
@@ -124,54 +128,59 @@ def is_talk_in_past(talk: dict, now: dt.datetime) -> bool:
     return now > end
 
 
+def format_row(row, column_width):
+    def format_cell(cell, column_width):
+        if type(cell) == str and (value_length := len(cell)) >= column_width:
+            cell = " \n".join(
+                # fmt: off
+                cell[n:n + column_width]
+                # fmt: on
+                for n in range(0, value_length, column_width)
+            )
+        return cell
+
+    return [format_cell(cell, column_width) for cell in row]
+
+
 def filter_talk(
-    talk: dict,
-    speaker: str,
-    title: str,
-    track: str,
-    day: int,
-    start: str,
-    room: str,
-    conference: str,
-    filter_past: bool,
-    now: dt.datetime,
+    talk: dict = {},
+    speaker: str = cli_defaults["speaker"],
+    title: str = cli_defaults["title"],
+    track: str = cli_defaults["track"],
+    day: int = cli_defaults["day"],
+    start: str = cli_defaults["start"],
+    room: str = cli_defaults["room"],
+    conference: str = cli_defaults["conference"],
+    filter_past: bool = cli_defaults["no_past"],
+    now: dt.datetime = dt.datetime.now().astimezone(),  # evaluated at function definition time, this is fine
 ) -> bool:
     """
     Some simple filter rules as functions
     """
 
-    def speaker_matches():
-        return speaker is None or speaker.lower() in talk["speakers"].lower()
-
-    def title_matches():
-        return title is None or title.lower() in talk["title"].lower()
-
-    def track_matches():
-        return track is None or track.lower() in talk["track"].lower()
+    def generic_in_match(filter_value, filter_key, talk_attribute):
+        return (
+            filter_value == cli_defaults[filter_key]
+            or filter_value.lower() in talk[talk_attribute].lower()
+        )
 
     def day_matches():
-        return day == 0 or day == talk["day"]
+        return day == -1 or day == talk["day"]
 
     def start_matches():
         return start is None or is_talk_in_timerange(talk, start)
 
-    def room_matches():
-        return room == "all" or room.lower() in talk["room"].lower()
-
-    def conference_matches():
-        return conference == "all" or conference == talk["conference_acronym"]
-
     def filtered_as_past():
         return filter_past and is_talk_in_past(talk, now)
 
     return (
-        speaker_matches()
-        and title_matches()
-        and track_matches()
+        generic_in_match(conference, "conference", "conference_acronym")
+        and generic_in_match(speaker, "speaker", "speakers")
+        and generic_in_match(title, "title", "title")
+        and generic_in_match(track, "track", "track")
         and day_matches()
         and start_matches()
-        and room_matches()
-        and conference_matches()
+        and generic_in_match(room, "room", "room")
         and not filtered_as_past()
     )
 
@@ -183,6 +192,7 @@ def print_formatted_talks(
     sort_by: str,
     reverse: bool,
     tablefmt: str,
+    column_width: int,
 ) -> None:
     header = [
         "Conference",
@@ -211,14 +221,14 @@ def print_formatted_talks(
         header.append("Description")
     data = []
 
-    for index, talk in enumerate(talks):
-        current_data_point = [talk[field] for field in fields]
+    for talk in talks:
+        current_data_point = [talk.get(field, "") for field in fields]
         if show_abstract:
             current_data_point.append(talk["talk_abstract"])
         if show_description:
             current_data_point.append(talk["talk_description"])
         # TODO think about coloring every second row (needs theming and config tho)
-        data.append(current_data_point)
+        data.append(format_row(current_data_point, column_width))
     try:
         if sort_by is not None:
             data.sort(key=lambda x: x[fields.index(sort_by)])
@@ -234,21 +244,31 @@ def print_formatted_talks(
 @click.option(
     "--title",
     "-t",
-    default=None,
+    default=cli_defaults["title"],
     help="A part of the title of the talk(s) you want to search.",
 )
 @click.option(
     "--track",
     "-tr",
-    default=None,
+    default=cli_defaults["track"],
     help="A part of the track description you want to search.",
 )
-@click.option("--day", "-d", default=0, help="Day you want to filter [1-4] or 0 for all days.")
-@click.option("--start", "-st", default=None, help="Start time of the talk(s) you want to search.")
+@click.option(
+    "--day",
+    "-d",
+    default=cli_defaults["day"],
+    help="Day you want to filter [1-4] or 0 for all days.",
+)
+@click.option(
+    "--start",
+    "-st",
+    default=cli_defaults["start"],
+    help="Start time of the talk(s) you want to search.",
+)
 @click.option(
     "--room",
     "-r",
-    default="all",
+    default=cli_defaults["room"],
     help="Name of the room you want to filter [room names] or 'all' for all rooms",
 )
 @click.option(
@@ -259,42 +279,45 @@ def print_formatted_talks(
 )
 @click.option(
     "--show-abstract",
-    default=False,
+    default=cli_defaults["show_abstract"],
     help="Shows abstracts, default False, experimental",
     is_flag=True,
 )
 @click.option(
     "--show-description",
-    default=False,
+    default=cli_defaults["show_description"],
     help="Shows descriptions, default False, experimental",
     is_flag=True,
 )
 @click.option(
     "--sort",
-    default=None,
+    default=cli_defaults["sort"],
     type=click.Choice(["day", "speakers", "title", "track", "room", "talk_start"]),
     help="Sort by day|speakers|title|track|room|talk_start",
 )
 @click.option("--reverse", default=False, help="Reverse results", is_flag=True)
 @click.option(
     "--tablefmt",
-    default="fancy_grid",
+    default=cli_defaults["tablefmt"],
     help="Choose a tableformat that is supported by python-tabular",
     type=click.Choice(_table_formats),
 )
 @click.option(
     "--column-width",
-    default=60,
+    default=cli_defaults["column_width"],
     help="Set the max width of the wide columns (which is everything string based)",
 )
 @click.option(
     "--update-cache",
-    default=False,
+    default=cli_defaults["update_cache"],
     help="Delete the cache file and redownload all fahrplans",
     is_flag=True,
 )
 @click.option(
-    "--no-past", default=False, help="Filter out talks that lay in the past", is_flag=True
+    "--no-past",
+    default=cli_defaults["no_past"],
+    help="Filter out talks that lay in the past",
+    is_flag=True,
 )
 def cli(
     speaker,
@@ -316,7 +339,7 @@ def cli(
     now = dt.datetime.now().astimezone()
     matching_talks = [
         x
-        for x in Fahrplan(column_width=column_width, update_cache=update_cache).flat_plans
+        for x in Fahrplan(update_cache=update_cache).flat_plans
         if filter_talk(x, speaker, title, track, day, start, room, conference, no_past, now)
     ]
     print_formatted_talks(
@@ -326,6 +349,7 @@ def cli(
         sort_by=sort,
         reverse=reverse,
         tablefmt=tablefmt,
+        column_width=column_width,
     )
 
 
